"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LanguageServicePlugin = void 0;
const Constants_cjs_1 = require("./Constants.cjs");
/**
 * Provides the functionality to validate files.
 */
class LanguageServicePlugin {
    /**
     * Initializes a new instance of the {@linkcode LanguageServicePlugin} class.
     *
     * @param pluginInfo
     * The context of the plugin.
     *
     * @param validator
     * A component for validating files.
     */
    constructor(pluginInfo, validator) {
        this.pluginInfo = pluginInfo;
        this.validator = validator;
    }
    /**
     * Gets or sets the context of the plugin.
     */
    get PluginInfo() {
        return this.pluginInfo;
    }
    /**
     * @inheritdoc
     */
    set PluginInfo(value) {
        this.pluginInfo = value;
    }
    /**
     * Gets the language service of the plugin.
     */
    get LanguageService() {
        if (!this.nameofLanguageService) {
            this.nameofLanguageService = this.Decorate();
        }
        return this.nameofLanguageService;
    }
    /**
     * Gets a component for validating files.
     */
    get Validator() {
        return this.validator;
    }
    /**
     * Gets the program of the transformation.
     */
    get Program() {
        return this.LanguageService.getProgram();
    }
    /**
     * Initializes the plugin.
     */
    Initialize() {
        this.nameofLanguageService = undefined;
    }
    /**
     * Refreshes the diagnostics.
     */
    Refresh() {
        this.PluginInfo.project.refreshDiagnostics();
    }
    /**
     * Decorates the language service if necessary.
     *
     * @returns
     * The decorated language service.
     */
    Decorate() {
        let languageService = this.PluginInfo.languageService;
        if (languageService[Constants_cjs_1.Constants.PluginInstalledSymbol]) {
            return languageService;
        }
        else {
            let proxy = Object.create(null);
            for (let key of Object.keys(languageService)) {
                const original = languageService[key];
                proxy[key] = (...args) => original.apply(languageService, args);
            }
            proxy.getSemanticDiagnostics = (fileName) => {
                let result = languageService.getSemanticDiagnostics(fileName);
                let program = this.Program;
                if (program) {
                    let sourceFile = program.getSourceFile(fileName);
                    if (sourceFile) {
                        result.push(...this.Validator.Validate(sourceFile, program.getCompilerOptions()));
                    }
                }
                return result;
            };
            return proxy;
        }
    }
}
exports.LanguageServicePlugin = LanguageServicePlugin;
//# sourceMappingURL=LanguageServicePlugin.cjs.map