"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypeScriptTransformerBase = void 0;
const common_1 = require("@typescript-nameof/common");
const TypeScriptAdapter_cjs_1 = require("./TypeScriptAdapter.cjs");
/**
 * Provides a basic implementation of a TypeScript transformer.
 *
 * @template TFeatures
 * The type of the features for the transformer.
 */
class TypeScriptTransformerBase extends common_1.TransformerBase {
    /**
     * Initializes a new instance of the {@linkcode TypeScriptTransformerBase} class.
     *
     * @param features
     * The features of the transformer.
     */
    constructor(features) {
        super(features);
    }
    /**
     * Gets a component for creating a transformer.
     */
    get Factory() {
        return (tsContext) => {
            return (file) => {
                return this.VisitSourceFile(file, tsContext);
            };
        };
    }
    /**
     * Gets a factory with a pre-defined {@linkcode context}.
     *
     * @param context
     * The context of the transformation.
     *
     * @returns
     * A newly created factory with a pre-defined {@linkcode context}.
     */
    GetFactory(context) {
        return (tsContext) => {
            return (file) => {
                return this.VisitSourceFile(file, tsContext, context);
            };
        };
    }
    /**
     * @inheritdoc
     *
     * @returns
     * The newly created adapter.
     */
    InitializeAdapter() {
        return new TypeScriptAdapter_cjs_1.TypeScriptAdapter(this.Features);
    }
    /**
     * Transforms the specified {@linkcode file}.
     *
     * @param file
     * The file to transform.
     *
     * @param tsContext
     * The context of the typescript transformation.
     *
     * @param context
     * The context of the transformation.
     *
     * @returns
     * The transformed representation of the specified {@linkcode file}.
     */
    VisitSourceFile(file, tsContext, context) {
        let defaults = context !== null && context !== void 0 ? context : {};
        return this.MonitorTransformation((context) => {
            let typeScriptContext = context;
            typeScriptContext.file = file;
            Object.assign(context, defaults);
            return this.VisitNode(file, typeScriptContext, tsContext);
        });
    }
    /**
     * Transforms the specified {@linkcode node}.
     *
     * @param node
     * The node to transform.
     *
     * @param context
     * The context of the operation.
     *
     * @param tsContext
     * The context of the typescript transformation.
     *
     * @returns
     * The transformed representation of the specified {@linkcode node}.
     */
    VisitNode(node, context, tsContext) {
        var _a, _b;
        node = this.Features.TypeScript.visitEachChild(node, (node) => {
            return this.VisitNode(node, context, tsContext);
        }, tsContext);
        let result = (_a = this.Adapter.Transform(node, context)) !== null && _a !== void 0 ? _a : node;
        (_b = context.postTransformHook) === null || _b === void 0 ? void 0 : _b.call(context, node, result);
        return result;
    }
}
exports.TypeScriptTransformerBase = TypeScriptTransformerBase;
//# sourceMappingURL=TypeScriptTransformerBase.cjs.map